@@ -1,7 +1,9 @@
 import asyncio
 
-from atlassian_jwt_auth.verifier import JWTAuthVerifier as _JWTAuthVerifier
+import jwt
+
 from atlassian_jwt_auth import key
+from atlassian_jwt_auth.verifier import JWTAuthVerifier as _JWTAuthVerifier
 
 
 class JWTAuthVerifier(_JWTAuthVerifier):
@@ -20,6 +22,8 @@ class JWTAuthVerifier(_JWTAuthVerifier):
         if asyncio.iscoroutine(public_key):
             public_key = await public_key
 
+        alg = jwt.get_unverified_header(a_jwt).get('alg', None)
+        public_key_obj = self._load_public_key(public_key, alg)
         return self._decode_jwt(
-            a_jwt, key_identifier, public_key,
+            a_jwt, key_identifier, public_key_obj,
             audience=audience, leeway=leeway)
